######################################################
#' @title Stacked Bar Plots to visualize PID MD Scores for CTC Pairs at CTC level
#' @author A. Grant Schissler
#' @date 11 Dec 2015
#' @description XXX
######################################################

######################################################
### i. Load packages and set up workspace
require(ggplot2)
setwd("./Figure5")
rm(list=ls())
## load data and patient information from CTC Pairs
load("../CommonData/CTC_pairs.RData")

######################################################
### 1. Format Data and Compute metrics

### first retrieve the actual MD Score
top.md.data <- data.frame(md.matrix)
names(top.md.data) <- gsub("^X","",names(top.md.data))

#### restrict to just B-A pairs
top.md.data <- top.md.data[b.a.index,]

#### restrict to the top five pathways
## top pathway IDs
top.ids <- c("200020", "200141", "200135", "200142", "200085")
top.labels <- c("ncWnt","ErbB2/B3","SDC4", "FOXM1", "S1P1")
## subset to only top five
top.md.data <- top.md.data[,top.ids]

## check for missing MD scores
any(is.na(top.md.data))

## retrieve pair names
pair.names <- rownames(top.md.data)

## parse the pair names to retrieve cell line label
ctc.labels <- unique(unlist(strsplit(pair.names,"-")))

### now retrieve the siginifance pathways
## count genesets under a certain FDR
threshold <- 0.05
type <- c("BY","p.value","Bon","Holm")
tmp.type <- type[2]

sig.indicator.list <- lapply(data.list, function(tmp.data) {
    tmp.data <- tmp.data[top.ids,]
    as.numeric(tmp.data[, tmp.type] < threshold & !is.nan(tmp.data$distance))
})

## create matrix for easy indexing
sig.indicator.matrix <- do.call("rbind",sig.indicator.list)
colnames(sig.indicator.matrix) <- top.ids

## restrict to B-A pairs
top.sig.matrix <- sig.indicator.matrix

any(is.na(top.sig.matrix))

## build data structure to populate
ctc.dys.data <- NULL
## populate with Dyregulation Status for a CTC
for (ii in ctc.labels) {
    ## retrieve patient
    tmp.patient <- strsplit(ii, "\\.")[[1]][1]
    ## treat A and B patients differently
    if (tmp.patient %in% a.patients) {
        ## reverse sign of dysregulation for A patients
        tmp.md.5 <- (-1)*top.md.data[ grep(paste0(ii,"$"), rownames(top.md.data)), ]
        ## inspect each pathway individually
        for (i in 1:length(top.ids)) {
            tmp.md <- tmp.md.5[,i]
            ## find the middle two pairs
            tmp.med.pairs <- rownames(tmp.md.5)[order(tmp.md)[c(length(tmp.md)/2,length(tmp.md)/2+1)]]
            #### define dysregulation type
            tmp.sig <- top.sig.matrix[tmp.med.pairs,i]
            tmp.sign <- sign(tmp.md.5[tmp.med.pairs,i])
            ### define each as +, -, or non-significant (NS)
            ### work through the cases
            ## case 1: both significant and same sign
            if (sum(tmp.sig) == 0) tmp.dys <- "NS"
            ## case 2: only one significant
            if (sum(tmp.sig) == 1) tmp.dys <- ifelse(tmp.sign[which(tmp.sig == 1)] > 0, yes = "+", no = "-")
            ## case 3: both significant and different sign
            if (sum(tmp.sig) == 2 & sum(tmp.sign) == 0) tmp.dys <- "NoCall"
            ## case 4: both significant and same sign
            if (sum(tmp.sig) == 2 & abs(sum(tmp.sign)) == 2) tmp.dys <- ifelse(tmp.sign[1] > 0, yes = "+", no = "-")
            ## store in temporary data frame
            tmp.data <- data.frame(CTC = ii, PID = top.ids[i], Dysregulation = tmp.dys, stringsAsFactors = F)
            ## store in long format overall data frame
            ctc.dys.data <- rbind(ctc.dys.data, tmp.data)
        }
        
    } else {
        ## B patients maintain direction and have exactly 1 median CTC pair
        tmp.md.5 <- top.md.data[ grep(paste0(ii,"[-|$]"), rownames(top.md.data)), ]
        ## find the median pairs
        tmp.med.pairs <- rownames(tmp.md.5)[apply(tmp.md.5, 2, function(tmp.id) {
            tmp.index <- which(tmp.id == median(tmp.id, na.rm = T))
        })]
        #### define dysregulation type
        tmp.sig <- top.sig.matrix[tmp.med.pairs,]
        ## retrieve MD score
        tmp.med.md <- tmp.md.5[tmp.med.pairs,]
        for (i in 1:length(top.ids)) {
            ## check significance for this PID
            if (tmp.sig[i,i] == 0) {
                ## Non-significance
                tmp.dys <- "NS"
            } else {
                ## significant, check direction
                tmp.dys <- ifelse(tmp.med.md[i,i] > 0, yes = "+", no = "-")
            }
            ## store in temporary data frame
            tmp.data <- data.frame(CTC = ii, PID = top.ids[i], Dysregulation = tmp.dys, stringsAsFactors = F)
            ## store in long format overall data frame
            ctc.dys.data <- rbind(ctc.dys.data, tmp.data)
        }
    }
}

## inspect data
str(ctc.dys.data)
tail(ctc.dys.data)

######################################################
### 2. Annotate with clinical data and create factors 

## retrieve patient labels
ctc.dys.data$Patient <- factor(unlist(lapply(strsplit(ctc.dys.data$CTC, "\\."), function(tmp) tmp[1])), levels = c(a.patients, b.patients))
ctc.dys.data$Group = factor(( as.character(ctc.dys.data$Patient) %in% b.patients), labels = c("Naive","Resistant"))
ctc.dys.data$Dysregulation <- factor(ctc.dys.data$Dysregulation, levels = c("-","NS","+"), labels = c("Down","Non-Sig","Up"))
ctc.dys.data$PID <- factor(ctc.dys.data$PID, levels = top.ids, labels = top.labels)

######################################################
### 3. Create stacked bar plot

p0 <- ggplot(ctc.dys.data, aes(x = Patient, fill = Dysregulation)) + geom_bar() + facet_grid(PID ~ .)

(p0 <- p0 + theme_bw() + scale_fill_brewer(type = "div", palette = 8) + theme(legend.position = "bottom") + scale_y_continuous(breaks=seq(0,max(12), by = 2)))

pdf("Figure5A.pdf", width = 7, height = 7)
p0 
dev.off()
