######################################################
#' @title Create Rose plot of Zscore
#' @author A. Grant Schissler
#' @date 6 Jan 2015
#' @description Inspiration for algorithm: http://stackoverflow.com/questions/10467988/modified-polar-plot-using-ggplots-or-other-alternative-packages-using-r, thanks to Mikko
######################################################

######################################################
### i. Load packages and set up workspace
require(ggplot2)
setwd("./Figure5")
rm(list=ls())
## load data and patient information from CTC Pairs
load("../CommonData/CTC_pairs.RData")
#### create custom color palettes
require("RColorBrewer")
### view all colors
## RColorBrewer::display.brewer.all()
(col.pos <- RColorBrewer::brewer.pal(9, "Blues")[3:7])
##
(col.neg <- RColorBrewer::brewer.pal(9, "YlOrBr")[3:7])

######################################################
### 1. Format Data and Compute metrics

### first retrieve the actual MD Score
top.md.data <- data.frame(md.matrix)
names(top.md.data) <- gsub("^X","",names(top.md.data))

#### restrict to just B-A pairs
top.md.data <- top.md.data[b.a.index,]

#### restrict to the top five pathways
## top pathway IDs
top.ids <- c("200020", "200141", "200135", "200142", "200085")
top.labels <- c("ncWnt","ErbB2/B3","SDC4", "FOXM1", "S1P1")
## subset to only top five
top.md.data <- top.md.data[,top.ids]

## check for missing MD scores
any(is.na(top.md.data))

## retrieve pair names
pair.names <- rownames(top.md.data)

## parse the pair names to retrieve cell line label
ctc.labels <- unique(unlist(strsplit(pair.names,"-")))

### now retrieve the siginifance pathways
## count genesets under a certain FDR
threshold <- 0.05
type <- c("BY","p.value","Bon","Holm")
tmp.type <- type[2]

sig.indicator.list <- lapply(data.list, function(tmp.data) {
 tmp.data <- tmp.data[top.ids,]
 as.numeric(tmp.data[, tmp.type] < threshold & !is.nan(tmp.data$distance))
})

## create matrix for easy indexing
sig.indicator.matrix <- do.call("rbind",sig.indicator.list)
colnames(sig.indicator.matrix) <- top.ids

## restrict to B-A pairs
top.sig.matrix <- sig.indicator.matrix
any(is.na(top.sig.matrix))

## build data structure to populate
ctc.dys.data <- NULL
## populate with Dyregulation Status for a CTC
for (ii in ctc.labels) {
    ## retrieve patient
    tmp.patient <- strsplit(ii, "\\.")[[1]][1]
    ## treat A and B patients differently
    if (tmp.patient %in% a.patients) {
        ## reverse sign of dysregulation for A patients
        tmp.md.5 <- (-1)*top.md.data[ grep(paste0(ii,"$"), rownames(top.md.data)), ]
        ## inspect each pathway individually
        for (i in 1:length(top.ids)) {
            tmp.md <- tmp.md.5[,i]
            ## find the middle two pairs
            tmp.med.pairs <- rownames(tmp.md.5)[order(tmp.md)[c(length(tmp.md)/2,length(tmp.md)/2+1)]]
            #### define dysregulation type
            tmp.sig <- top.sig.matrix[tmp.med.pairs,i]
            tmp.sign <- sign(tmp.md.5[tmp.med.pairs,i])
            ### define each as +, -, or non-significant (NS)
            ### work through the cases
            ## case 1: both insignificant and same sign
            if (sum(tmp.sig) == 0) tmp.dys <- "NS"
            ## case 2: only one significant
            if (sum(tmp.sig) == 1) tmp.dys <- ifelse(tmp.sign[which(tmp.sig == 1)] > 0, yes = "+", no = "-")
            ## case 3: both significant and different sign
            if (sum(tmp.sig) == 2 & sum(tmp.sign) == 0) tmp.dys <- "NoCall"
            ## case 4: both significant and same sign
            if (sum(tmp.sig) == 2 & abs(sum(tmp.sign)) == 2) tmp.dys <- ifelse(tmp.sign[1] > 0, yes = "+", no = "-")
            ## determine pvalues for the two median pairs for current pathway
            tmp.pvalue1 <- data.list[[tmp.med.pairs[1]]][top.ids[i],tmp.type]
            tmp.pvalue2 <- data.list[[tmp.med.pairs[2]]][top.ids[i],tmp.type]
            ## take smaller of the two pvalues
            tmp.pvalue <- min(tmp.pvalue1, tmp.pvalue2)
            ## determine direction for plotting of transformed pvalue (and print warning if signs are different
            if ( sum(tmp.sign) == 0) {
                warning(paste0("Median pairs for CTC ",ii, " have different direction of DEP."))
                tmp.direction <- "NoCall"
            } else {
                tmp.direction <- sign(sum(tmp.sign))
            }
            ## store in temporary data frame
            tmp.data <- data.frame(CTC = ii, Patient = tmp.patient, PID = top.ids[i], Direction = tmp.direction,
                                   Dysregulation = tmp.dys, Pvalue = tmp.pvalue, stringsAsFactors = F)
            ## store in long format overall data frame
            ctc.dys.data <- rbind(ctc.dys.data, tmp.data)
        }
        
    } else {
        ## B patients maintain direction and have exactly 1 median CTC pair
        tmp.md.5 <- top.md.data[ grep(paste0(ii,"[-|$]"), rownames(top.md.data)), ]
        ## find the median pairs
        tmp.med.pairs <- rownames(tmp.md.5)[apply(tmp.md.5, 2, function(tmp.id) {
            tmp.index <- which(tmp.id == median(tmp.id, na.rm = T))
        })]
        #### define dysregulation type
        tmp.sig <- top.sig.matrix[tmp.med.pairs,]
        ## retrieve MD score
        tmp.med.md <- tmp.md.5[tmp.med.pairs,]
        for (i in 1:length(top.ids)) {
            ## check significance for this PID
            if (tmp.sig[i,i] == 0) {
                ## Non-significance
                tmp.dys <- "NS"
            } else {
                ## significant, check direction
                tmp.dys <- ifelse(tmp.med.md[i,i] > 0, yes = "+", no = "-")
            }
            ## determine pvalue for the current pair and pathway
            tmp.pvalue <- data.list[[tmp.med.pairs[i]]][top.ids[i],tmp.type]
            ## store in temporary data frame
            tmp.data <- data.frame(CTC = ii, Patient = tmp.patient, PID = top.ids[i], Direction = sign(tmp.med.md[i,i]),
                                   Dysregulation = tmp.dys, Pvalue = tmp.pvalue, stringsAsFactors = F)
            ## store in long format overall data frame
            ctc.dys.data <- rbind(ctc.dys.data, tmp.data)
        }
    }
}

## annotate pathways
ctc.dys.data$Pathway <- factor(ctc.dys.data$PID, levels = top.ids, labels = top.labels)
## transform pvalue to plot (make sure to take square root later for representative petal areas
ctc.dys.data$tPvalue <- -log( ctc.dys.data$Pvalue, base = exp(1) )

## inspect data
str(ctc.dys.data)
tail(ctc.dys.data)


######################################################
### 2. Plot by patient
    
for (tmp.patient in c(a.patients,b.patients)) {
    ## plot all CTCs for one patient
    ## create long data to plot later
    rose.long <- NULL
    order.long <- NULL
    ## iterate through the cells
    tmp.ctc.all <- ctc.dys.data[ctc.dys.data$Patient == tmp.patient,"CTC"]
    for (tmp.id in unique(tmp.ctc.all)) {
        ## construct data frame for this CTC
        ctc.tPvalue.data <- data.frame(PID = unique(ctc.dys.data$Pathway),
                                       tPvalue = ctc.dys.data$tPvalue[ctc.dys.data$CTC == tmp.id],
                                       Direction = ctc.dys.data$Direction[ctc.dys.data$CTC == tmp.id])
        ## automatically order the plots
        order.data <- data.frame(CTC = tmp.id, tPvalue = mean(ctc.tPvalue.data$tPvalue*ctc.tPvalue.data$Direction),
                                 count.down = sum(ctc.tPvalue.data$Direction < 0))
        ## retrieve count of pathways
        pathways.num <- nrow(ctc.tPvalue.data)
        ## construct the bins for the position of the bars
        pos <- seq(0, pathways.num*2 - 1, by = 1)
        ## contstruct data structure for plotting
        rose.data <- data.frame(pathway = factor(1:(pathways.num*2)), tPvalue = 0, pos = pos)
        ## choose positions based on desired location up or down
        for (j in 1:nrow(ctc.tPvalue.data)) {
            ifelse( ctc.tPvalue.data$Direction[j] > 0,
                   rose.data$tPvalue[j] <- ctc.tPvalue.data$tPvalue[j],
                   rose.data$tPvalue[j + pathways.num] <- ctc.tPvalue.data$tPvalue[j])
        }
        
        ## transform tPvalue scores (radii) to maintain equal area instead of radii
        rose.data$tPvalue <- sqrt(rose.data$tPvalue)
        ## store order data
        order.long <- rbind(order.long,order.data)
        ## store rose data
        rose.data$CTC <- tmp.id
        rose.long <- rbind(rose.long, rose.data)
    }

    ## order CTCs by average tPvalue score to see patterns
    ctc.levels <- as.character(order.long[order(order.long$count.down, order.long$tPvalue), "CTC"])
    
    ## make CTC a factor
    rose.long$CTC <- factor(rose.long$CTC, levels = ctc.levels)
    rose.long$pathway <- factor( rose.long$pathway, labels = c(paste0(top.labels," +"),paste0(top.labels," -") ))

    ## create ggplot object
    p0 <- ggplot(rose.long, aes(fill = factor(pathway), x = pos, y = tPvalue))
    ## create bar plot to later convert to polar coordinates
    width <- 1
    p1 <- p0 + geom_bar(width = width, stat = "identity")
    ## create rose and shift pathways to start from horizontal axis
    p2 <- p1 + coord_polar(start = -pi*(nrow(rose.data)) - pi/2)
    ## update y scales to show breaks of interest
    ## p3 <- p2 + scale_x_continuous(labels = NULL, breaks = pos + width/2, minor_breaks = NULL)
    p3 <- p2 + scale_x_continuous(labels = NULL, breaks = (pos + width/2)[c(5,10)], minor_breaks = NULL)
    ## customize fill scale
    ## p4 <- p3 + scale_fill_manual(values = rep(lab.colors[5:1], times = 2), guide = guide_legend(title = "Pathways"))
    p4 <- p3 + scale_fill_manual(values = c(col.pos,col.neg), guide = guide_legend(title = "Pathways"))
    ## customize theme
    p5 <- p4 + theme_bw() + xlab("")
    ## add darker horizontal lines to guide eye for up vs. down regulation
    p6 <- p5 + theme(panel.grid.major.x = element_line(colour = "black", linetype = 1))
    ## update y scales to show breaks of interest (significance levels)
    ybreaks <- sqrt (-log(c(0.05, 0.01, 0.001), exp(1)) )
    p7 <- p6 + scale_y_continuous(labels = NULL, breaks = ybreaks, minor_breaks = NULL) +
        theme(panel.grid.major.y = element_line(colour = "black", linetype = 1))
    ## facet by Cell
    p8 <- p7 + facet_wrap(~ CTC)
    ## update y scales labeling
    p9 <- p8 + theme(axis.ticks = element_blank(), axis.text.y = element_blank(), axis.title.y = element_blank())
    ## remove dark color from background
    p10 <- p9 + theme(strip.background = element_rect(fill = 'white'))
    ## move legend, manually move the legend in Adobe illustrator to place precisely
    p11 <- p10 + theme(legend.position = "bottom", legend.direction = "vertical")

    ## use ggsave to avoid problems with shapes
    ggsave(plot=p11 ,height=20, width=20, dpi=300,
           filename=paste0(tmp.patient,"_Rose_tPvalue.pdf"), useDingbats=FALSE)

}

